-------------------------------------------------------------------------------
--! @file       MCN.vhd
--! @author     Ahmed Ferozpuri
--! @brief      MixColumns Generator Component for PRIMATEs
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY MCN IS
	GENERIC (N : INTEGER := 5;
				V : INTEGER := 200;
				isINV : INTEGER := 0);
	PORT(	V_i : IN  std_logic_vector(V-1 DOWNTO 0);
			V_o : OUT  std_logic_vector(V-1 DOWNTO 0)
		);
END MCN;

ARCHITECTURE behavioral OF MCN IS

	TYPE tType IS ARRAY (0 TO N) OF std_logic_vector (V-1 DOWNTO 0); 
	SIGNAL T : tType;
BEGIN
	
	T(0) <= V_i;
	V_o <= T(N);
	MCN:IF V=200 OR V=280 GENERATE
		GEN_MCN: FOR i IN 0 TO N-1 GENERATE
			MC:	ENTITY work.MC(behavioral)
					GENERIC MAP (C => N*5, V => V, isINV => isINV)
					PORT MAP(	V_i => T(i), V_o => T(i+1) );
		END GENERATE GEN_MCN;
	END GENERATE MCN;
END behavioral;