-------------------------------------------------------------------------------
--! @file       PN.vhd
--! @author     Ahmed Ferozpuri
--! @brief      Permutation for PRIMATEs (also includes inverse)
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY PN IS
	GENERIC (V : INTEGER := 200;
				isINV : INTEGER := 0);
	PORT(	clk, init, run, mode: IN std_logic;
			pn : IN std_logic_vector(1 downto 0);
			V_i : IN  std_logic_vector(V-1 DOWNTO 0);
			done : OUT std_logic;
			V_o : OUT  std_logic_vector(V-1 DOWNTO 0)
		);
END PN;

ARCHITECTURE behavioral OF PN IS
	
	SIGNAL ViReg :std_logic_vector(V-1 DOWNTO 0);
	SIGNAL VoReg, VoSB, VoSR, VoMC, VoCA, VoP : std_logic_vector(V-1 DOWNTO 0);
	SIGNAL VoSBInv, VoSRInv, VoMCInv, VoCAInv : std_logic_vector(V-1 DOWNTO 0);
	SIGNAL DecIn : std_logic_vector(3 downto 0);
	SIGNAL DecOut : std_logic_vector(3 downto 0);
	SIGNAL selD, enV, donel, ldR : std_logic;
BEGIN

		enV <= (run OR init) AND (NOT donel);
		--ldR <= init OR done;
		ViReg <= V_i WHEN init = '1' ELSE VoP;
		selD <= pn(1) XOR pn(0);
		DecIn <= "0110" WHEN selD = '1' ELSE "1100";
		donel <= '1' WHEN DecOut = "0000" ELSE '0';
		done <= donel;
		V_o <= VoReg;
		
		DEC4_1: ENTITY work.DECN_M(behavioral)
			GENERIC MAP (N => 4, M => 1) --dec 1 from a 4 bit counter
			PORT MAP(	clk => clk,	en => run, ld => init, D => DecIn,	Q => DecOut);
			
		REGV: ENTITY work.REGN_aqf(behavioral)
			GENERIC MAP (N => V)
			PORT MAP(	clk => clk,	en => enV, D => ViReg,	Q => VoReg);
		
		SB: ENTITY work.SB(behavioral)
			GENERIC MAP (V => V, isINV => 0)
			PORT MAP(	V_i => VoReg,	V_o => VoSB);
		
		SR: ENTITY work.SR(behavioral)
			GENERIC MAP (V => V, isINV => 0)
			PORT MAP(	V_i => VoSB,	V_o => VoSR);

		MC5:IF V=200 GENERATE
			MCN: ENTITY work.MCN(behavioral)
				GENERIC MAP (N=>5, V => V)
				PORT MAP(	V_i => VoSR,	V_o => VoMC);
		END GENERATE MC5;
		
		MC7:IF V=280 GENERATE
			MCN: ENTITY work.MCN(behavioral)
				GENERIC MAP (N=>7, V => V)
				PORT MAP(	V_i => VoSR,	V_o => VoMC);
		END GENERATE MC7;
		
		CA: ENTITY work.CA(behavioral)
			GENERIC MAP (V => V, isINV => 0)
			PORT MAP(	clk => clk, enLFSR => enV,	selLFSR => init, pn => pn, V_i => VoMC, V_o => VoCA);
			
		GENPN:IF isINV=0 GENERATE	
			VoP <= VoCA;
		END GENERATE GENPN;
		
		GENPNINV:IF isINV=1 GENERATE
			CAINV: ENTITY work.CA(behavioral)
				GENERIC MAP (V => V, isINV => 1)
				PORT MAP(	clk => clk, enLFSR => enV,	selLFSR => init, pn => pn, V_i => VoReg, V_o => VoCAInv);

			MC5:IF V=200 GENERATE
				MCN: ENTITY work.MCN(behavioral)
					GENERIC MAP (N=>5, V => V, isINV => 1)
					PORT MAP(	V_i => VoCAInv,	V_o => VoMCInv);
			END GENERATE MC5;
			
			MC7:IF V=280 GENERATE
				MCN: ENTITY work.MCN(behavioral)
					GENERIC MAP (N=>7, V => V, isINV => 1)
					PORT MAP(	V_i => VoCAInv,	V_o => VoMCInv);
			END GENERATE MC7;
				
			SRINV: ENTITY work.SR(behavioral)
				GENERIC MAP (V => V, isINV => 1)
				PORT MAP(	V_i => VoMCInv,	V_o => VoSRInv);
				
			SBINV: ENTITY work.SB(behavioral)
				GENERIC MAP (V => V, isINV => 1)
				PORT MAP(	V_i => VoSRInv,	V_o => VoSBInv);
				
			VoP <= VoCA WHEN mode='0' ELSE VoSBInv;
		END GENERATE GENPNINV;
END behavioral;

